#!/usr/bin python
#*****************************************************************************
#                           IPG Converter
#
#           Python script for IPG Data Conversion and Analysis 
#              written by Mark S. Adams, Cisco Lawrenceville
#
#                  Copyright 2011, Cisco Systems, Inc.
#
#                     Proprietary and Confidential
#         Unauthorized distribution or copying is prohibited
#                          All rights reserved
#
# No part of this computer software may be reprinted, reproduced or utilized
# in any form or by any electronic, mechanical, or other means, now known or
# hereafter invented, including photocopying and recording, or using any
# information storage and retrieval system, without permission in writing
# from Cisco Systems, Inc.
#*****************************************************************************

import os
import sys
import logging
from optparse import OptionParser

import re
import time
import gzip
import zipfile

Version = '0.79'

# Truncation option is currently ON
#truncate_call_letters_to_five_characters = False
truncate_call_letters_to_five_characters = True

EpisodeCounter = 0

class IpgDataItem(object):
    def __init__(self, list):
        self.list = list

class IpgDataList(object):
    def __init__(self):

        data_tags_as_text = '''
            cisco_CName
            cisco_Date
            cisco_Time
            cisco_Duration
            cisco_STitle
            cisco_LTitle
            cisco_Theme
            cisco_SDesc
            cisco_LDesc
            cisco_MpaaRating
            cisco_TvRating
            cisco_Stereo
            cisco_Surround
            cisco_Sap
            cisco_ClosedCaptioned
            cisco_Animated
            cisco_BlackWhite
            cisco_ReRun
            cisco_Live
            cisco_IsMovie
            cisco_Nudity
            cisco_Language
            cisco_Violence
            cisco_AdultTheme
            cisco_HalfStars
            cisco_Hd
            vid_VideotronCallLetters
            vid_ChannelDefinition
            vid_StartTime
            vid_Duration
            vid_ProgramId
            vid_Title
            vid_Block1
            vid_Category
            vid_Subcategory
            vid_SeriesId
            vid_EpisodeTitle
            vid_EpisodeId
            vid_Rating
            vid_Sap
            vid_ProductionYear
            vid_Country
            vid_FullDescription
            vid_Actors
            parse_index
            '''
        # Build tags and make as attributes
        data_tags = []
        for text in data_tags_as_text.split('\n'):
            tag = text.strip()
            if tag:
                data_tags.append(tag)
                self.__dict__[ tag ] = tag
        self.set_convertion_data()
        
        self.config_cisco_file     = ''
        self.config_videotron_file = ''
        self.config_overlap        = 0
        self.config_report         = 0
        self.errors = []
        self.content_in = ''
        self.content_out = ''
        self.log_obj = None
        self.test_mode_quick = False
        self.TEST_MODE_QUICK_COUNT = 10
        self.reset_conversion()
        self.histo_rating_mpaa = {}
        self.histo_rating_tv = {}
        self.mpaa_ratings_available  = ['', 'MPAAG','MPAAPG','MPAAPG13','MPAAR','MPAANC17','NR','NR-ADULT']
        self.tv_ratings_available    = ['', 'TVY','TVY7','TVG','TVPG','TV14','NR-ADULT','TVMA']
        self.cisco_to_videotron_themes_per_F0330 = {
            0 :  0,
            1 : 15,
            2 :  6,
            3 :  8,
            4 : 13,
            5 : 16,
            6 : 16,
            7 : 17,
            8 : 23,
            9 : 24,
            10: 36,
            11: 26,
            12: 34,
            13:  5,
            14:  4,
            15: 14,
            16: 22,
            17: 11,
            18: 19,
            19: 18
            }

        self.cisco_to_videotron_themes = {
            0 :  [13],
            1 :  [ 0],
            2 :  [ 1],
            3 :  [ 3],
            4 :  [ 2],
            5 :  [ 6],
            6 :  [ 2],
            7 :  [ 3],
            8 :  [ 4],
            9 :  [ 6],
            10:  [10],
            11:  [13],
            12:  [ 3],
            13:  [ 5],
            14:  [10],
            15:  [13], # Says NA ?
            16:  [ 9],
            17:  [ 8],
            18:  [10],
            19:  [11],
            20:  [ 2, 9],
            21:  [13],
            22:  [10],
            23:  [ 4],
            24:  [ 7],
            25:  [ 4],
            26:  [12],
            27:  [12],
            28:  [12],
            29:  [12],
            30:  [12],
            31:  [12],
            32:  [12],
            33:  [12],
            34:  [13],
            35:  [13],
            36:  [0]
            }

        
    #####################################################################################
    #   General Functions
    #####################################################################################
        
    def info(self, text):
        if self.log_obj:
            self.log_obj.info(text)
        else:
            print('Info: %s'%(text))
            

    def reset_conversion(self):
        # One or the other, Report or Analysis
        self.conversion_list      = []
        self.conversion_warnings  = 0
        self.conversion_errors    = 0
        self.conversion_on        = True
        self.overlap_on           = False


    def reset_overlap(self):
        self.overlap_list         = []
        self.overlap_warnings     = 0
        self.overlap_errors       = 0
        self.overlap_on           = True
        self.conversion_on        = False


    def report_info(self, text):
        report = 'Info: %s'%(text)
        if self.conversion_on:
            self.conversion_list.append(report)
        if self.overlap_on:
            self.overlap_list.append(report)
        if self.log_obj:
            self.log_obj.info(text)
        else:
            print('Info: %s'%(text))


    def warning(self, text):
        report = 'W%3d: %s'%(self.conversion_warnings+1, text)
        if self.conversion_on:
            self.conversion_warnings += 1
            self.conversion_list.append(report)
        if self.overlap_on:
            self.overlap_warnings += 1
            self.overlap_list.append(report)
            #self.overlap_list.append('Warning above from file: %s'%(self.filename_source))
        if self.log_obj:
            self.log_obj.warning(text)
        else:
            pass
            # Only print if -r option on in CLI mode
            #print('Warning %s'%(text))


    def error(self, text):
        report = 'E%3d: %s'%(self.conversion_errors+1, text)
        if self.conversion_on:
            self.conversion_errors += 1
            self.conversion_list.append(report)
        if self.overlap_on:
            self.overlap_errors += 1
            self.overlap_list.append(report)
            #self.overlap_list.append('Error above from file: %s'%(self.filename_source))
        if self.log_obj:
            self.log_obj.error(text)
        else:
            print('Error : %s'%(text))
            

    def get_name_no_extension(self, name ):
        name_no_extension = name
        parts = name.split('.')
        if len(parts) == 2:
            name_no_extension = parts[0]
        if  len(parts) > 2:
            name_no_extension = '.'.join(parts[0:-1])
        return name_no_extension
    

    def get_extension(self, name ):
        extension = ''
        parts = name.split('.')
        if  len(parts) > 1:
            extension = parts[-1]
        return extension


    def current_time_iso_8601(self, textIn=0, tmIn=0 ):
        # ISO 8601
        if tmIn:
            tm = tmIn
        else:
            tm = time.localtime()
        current = '%4d%02d%02dT%02d%02d%02d'%( int(tm[0]), int(tm[1]), int(tm[2]), int(tm[3]), int(tm[4]), int(tm[5]) )
        if textIn:
            reDate = re.compile( r'(.*)([0-9]{8}T[0-9]{4}[0-9]{2})(.*)' )
            sea = re.search( reDate, textIn)
            if sea:
                newdate = '%s%s%s'%(sea.group(1), current, sea.group(3) )
            else:
                newdate = '%s_%s'%(current, textIn)
        else:
            newdate = current
        return newdate

    
    def get_dated_filename(self, filename, added_text=''):
        basename  = os.path.basename(filename)
        extension = self.get_extension(basename)
        name      = self.get_name_no_extension(basename)
        extension_t = ''
        if extension:
            extension_t = '.' + extension
        new_name  = self.current_time_iso_8601(name) + added_text + extension_t
        return( os.path.join(os.path.dirname(filename), new_name))


    def get_new_filename_dated_if_needed(self, file, extention=0):
        if extention:
            new_file = file + '.' + extention
        else:
            new_file = file
        if os.path.exists(new_file):
            new_file = self.get_dated_filename(new_file)
        return(new_file)            


    def make_time_from_seconds(self, secs_in):
        text = ''
        if secs_in > 24 * 60 * 60:
            days = int( secs_in / (24 * 60 * 60))
            text += '%d days '%(days)
            secs_in -= days * 24 * 60 * 60
        if secs_in > 60 * 60:
            hours = int( secs_in / (60 * 60 ))
            text += '%d hours '%(hours)
            secs_in -= hours * 60 * 60
        if secs_in > 60:
            mins = int( secs_in / 60 )
            text += '%d mins '%(mins)
            secs_in -= mins * 60 
        text += '%d seconds '%(secs_in)
        return text
            

    def check_filename(self, file):
        if os.path.exists(file):
            return 0
        self.error('File does not exists: %s'%(file))
            #sys.exc_info()[0], file))
        return 1


    #####################################################################################
    #  File Functions
    #####################################################################################
        
    def load_text_file(self, filename):
        if self.check_filename(filename):
            return 1
        self.filename_source = filename
        self.report_info('Loading Text file: %s'%(self.filename_source))
        self.text_file = open(filename, 'r')
        self.content_in = self.text_file.read()
        self.text_file.close()
        return 0


    def load_gzip_file(self, filename):
        if self.check_filename(filename):
            return 1
        self.filename_source = filename
        self.report_info('Loading GZip file: %s'%(self.filename_source))
        self.gzip_file = gzip.open(filename, 'rb')
        self.content_in = self.gzip_file.read()
        self.gzip_file.close()
        return 0


    def extractall_zip_file(self, filename):
        if self.check_filename(filename):
            return 1
        self.filename_source = filename
        self.report_info('Loading Zip file: %s'%(self.filename_source))
        self.zip_file = zipfile.ZipFile(filename, 'r')
        self.zip_file.extractall(os.path.dirname(filename))
        self.zip_file.close()
        return 0


    def save_content_as_text(self, filename):
        # We use b-binary mode so Windows does not add <returns> per <linefeed>
        self.filename_dest  = filename
        self.report_info('Saving Text file: %s'%(self.filename_dest))
        self.text_file = open(filename, 'wb')
        self.text_file.write(self.content_out)
        self.text_file.close()
        return 0


    def save_content_as_gzip(self, filename):
        self.filename_dest  = filename
        self.report_info('Saving GZip file: %s'%(self.filename_dest))
        self.gzip_file = gzip.open(filename, 'wb')
        self.gzip_file.write(self.content_out)
        self.gzip_file.close()
        return 0


    def save_content_as_zip(self, filename, arcname):
        self.filename_dest  = filename
        self.report_info('Saving Zip file: %s'%(self.filename_dest))
        self.zip_file = zipfile.ZipFile(filename, 'w', zipfile.ZIP_DEFLATED)
        self.zip_file.writestr(arcname, self.content_out)
        self.zip_file.close()
        return 0
            
    def save_content_list_as_zip(self, filename, arcname_list, content_list ):
        self.filename_dest  = filename
        self.report_info('Saving Zip file: %s'%(self.filename_dest))
        self.zip_file = zipfile.ZipFile(filename, 'w', zipfile.ZIP_DEFLATED)
        for i, arcname in enumerate(arcname_list):
            self.zip_file.writestr(arcname, content_list[i])
        self.zip_file.close()
        return 0
    
        notes = '''
        compression is the ZIP compression method to use when 
        writing the archive, and should be ZIP_STORED or ZIP_DEFLATED;
       zipfile.ZipFile ZIP_DEFLATED        
        '''
            

    def save_files_as_zip(self, filename, files):
        self.zip_file = zipfile.ZipFile(filename, 'w', zipfile.ZIP_DEFLATED)
        for file in files:
            self.zip_file.write(file, os.path.basename(file))
        self.zip_file.close()
        return 0
        

    #####################################################################################
    #   CLI Functions
    #####################################################################################
        
    def run_from_cli(self):
        start_time = time.time()
        print '\nScript version %s started: %s'%(Version, time.ctime())
        print 'Script: %s'%(' '.join(sys.argv))
        if self.config_overlap:
            self.run_from_cli_overlap()
        else:
            self.run_from_cli_convert()
        print 'Script ended: %s, Elapsed time: %s\n'%(
            time.ctime(), self.make_time_from_seconds(time.time() - start_time + 1))
            
    def run_from_cli_overlap(self):
        if self.config_videotron_file:
            videotron_format = 1
            file = self.config_videotron_file
        elif self.config_cisco_file:
            videotron_format = 0
            file = self.config_cisco_file
        else:
            self.error('Need either Cisco and Videotron filenames to run Overlap Analysis.')
            return 1
        self.reset_overlap()
        start = time.time()
        ext = self.get_extension(file).lower()
        if ext == 'gz':
            if self.load_gzip_file(file):
                return 1
        elif ext == 'zip' :
            self.error('  !!! Please do not select ".zip" files: %s.'%(
                os.path.basename(file)))
            return 1
        else:
            if self.load_text_file(file):
                return 1
        self.ipg_overlap_analysis(videotron_format)
        duration = time.time() - start 
        no_error_text = 'Warnings (%d), Errors (%d)'%(self.overlap_warnings, self.overlap_errors)
        if not self.overlap_warnings and not self.overlap_errors:
            no_error_text = ', no warnings or errors.'
        self.info('Overlap Analysis made in %d minutes and %d seconds %s'%(
            duration / 60, duration % 60, no_error_text))
        if 0:
            if self.overlap_warnings or self.overlap_errors:
                if len( self.overlap_list ):
                    self.info('Overlap Report for file: %s'%(file))
                    for line in self.overlap_list:
                        if line[0].lower() == 'w':
                            print( 'Warning: ' + line)
                        elif line[0].lower() == 'e':
                            print( 'Error: ' + line)
                        else:
                            print(line)
                    self.info('End of Overlap Report:')
                else:
                    self.info('Overlap Report: No Warnings, No Errors.')

           
    def run_from_cli_convert(self):
        file_in  = 0 
        dir_out = 0 
        if self.config_cisco_file:
            file_in = self.config_cisco_file
        if self.config_videotron_file:
            dir_out = self.config_videotron_file
        if not file_in or not dir_out:
            self.error('Need both Cisco and Videotron filenames to run Conversion.')
            return 1
        self.reset_conversion()
        start = time.time()
        ext = self.get_extension(file_in).lower()
        if ext == 'gz':
            if self.load_gzip_file(file_in):
                return 1
        else:
            if self.load_text_file(file_in):
                return 1
        self.load_time_slots(videotron_format=0)
        self.make_time_windows()
        self.save_days(dir_out)         
        duration = time.time() - start 
        no_error_text = 'Warnings (%d), Errors (%d)'%(
            self.conversion_warnings, self.conversion_errors)
        if not self.conversion_warnings and not self.conversion_errors:
            no_error_text = ', no warnings or errors.'
        self.info('Converted in %d minutes and %d seconds %s'%(
            duration / 60, duration % 60, no_error_text))
        if self.conversion_warnings:
            self.info('!!! There were %d Warnings, use CLI option "-w1" to see them.'%(
                self.conversion_warnings))
        if self.conversion_errors:
            self.info('!!! There were %d Errors'%(
                self.conversion_errors))
        if self.config_report and (self.conversion_warnings or self.conversion_errors):
            if len( self.conversion_list ):
                self.info('Conversion Report:')
                for line in self.conversion_list:
                    if line[0].lower() == 'w':
                        print( 'Warning: '+ line)
                    elif line[0].lower() == 'e':
                        print( 'Error: ' + line)
                    else:
                        print(line)
                self.info('End of Conversion Report.')
            else:
                self.info('Conversion Report: No Warnings, No Errors.')

        
        
    #####################################################################################
    #   Cisco / SA IPG to Videotron Conversion Functions
    #####################################################################################

    def parse_one_line(self, parse_index, line_in, delimiter, tags, cisco_mode=1 ):
        line = line_in.strip()
        parts = line.split(delimiter)
        if cisco_mode:
            # Cisco format adds an extra delimiter, remove last item
            parts = parts[:-1]
        program_items = { self.parse_index : parse_index }
        if len(parts) != len(tags):
            self.error('Line %7d: Wrong item count (%d actual %d needed)'%(parse_index,len(parts), len(tags)))
            return(program_items)
        for i,tag in enumerate( tags):
            program_items[tag] = parts[i]
        return(program_items)


    def parse(self, content_in, delimiter, tags, cisco_mode=1 ):
        data = []        
        self.parse_lines = content_in.split('\n')
        for i, line in enumerate( self.parse_lines ):
            if len(line.split(delimiter)) < 2:
                continue
            program_items = self.parse_one_line(i, line, delimiter, tags, cisco_mode)
            data.append( program_items )
        return data
    
    
    def set_convertion_data(self):
        self.cisco_parse_tags = [
            self.cisco_CName,
            self.cisco_Date,
            self.cisco_Time,
            self.cisco_Duration,
            self.cisco_STitle,
            self.cisco_LTitle,
            self.cisco_Theme,
            self.cisco_SDesc,
            self.cisco_LDesc,
            self.cisco_MpaaRating,
            self.cisco_TvRating,
            self.cisco_Stereo,
            self.cisco_Surround,
            self.cisco_Sap,
            self.cisco_ClosedCaptioned,
            self.cisco_Animated,
            self.cisco_BlackWhite,
            self.cisco_ReRun,
            self.cisco_Live,
            self.cisco_IsMovie,
            self.cisco_Nudity,
            self.cisco_Language,
            self.cisco_Violence,
            self.cisco_AdultTheme,
            self.cisco_HalfStars,
            self.cisco_Hd,
            ]

        self.videotron_parse_tags = [
            self.vid_VideotronCallLetters,
            self.vid_ChannelDefinition,
            self.vid_StartTime,
            self.vid_Duration,
            self.vid_ProgramId,
            self.vid_Title,
            self.vid_Block1,
            self.vid_Category,
            self.vid_Subcategory,
            self.vid_SeriesId,
            self.vid_EpisodeTitle,
            self.vid_EpisodeId,
            self.vid_Rating,
            self.vid_Sap,
            self.vid_ProductionYear,
            self.vid_Country,
            self.vid_FullDescription,
            self.vid_Actors,
            ]

    def check_size(self, item, size, text, linenumber):
        if len(item) != size:
            self.error('Line %7d: %s <%s> has length %d but it should be %d.'%(
                linenumber, text, item, len(item), size))
            return 1
        return 0


    def create_id_from_text(self, title):
        tid = 0
        sparse = max( len(title)/5, 1)
        for i in range(4):
            j = i*sparse
            if j < len(title):
                tid <<= 6
                tid += ord(title[j])
        for i,cc in enumerate(title):
                tid += ord(cc) + i
        return '%d'%(tid)

    
    def is_on(self, data, tag ):
        # We assume no data to be a False
        return( int(data.get(tag, '').upper() == 'Y')) 


    def get_int(self, data, tag ):
        # We assume no data to be a 0
        n = 0
        try:
            n = int(data.get(self.tag, '0'))
        except:
            pass
        return(n) 
    
            
    def convert_one_cisco_program_to_videotron(self, cdata, linenumber):
        global EpisodeCounter
        vdata = {}
        EpisodeCounter += 1
        
        # vid_VideotronCallLetters
        call_letters = cdata[self.cisco_CName]
        if truncate_call_letters_to_five_characters and len(call_letters) > 5:
            call_letters = call_letters[:5]
            self.warning('Line %7d: Call Letters (%s) will be truncated to (%s)'%(
                linenumber, cdata[self.cisco_CName], call_letters))
        vdata[ self.vid_VideotronCallLetters ] = call_letters
        
        # vid_ChannelDefinition
        vdata[ self.vid_ChannelDefinition ] = ''
        
        # vid_StartTime
        cisco_date = cdata[self.cisco_Date]
        cisco_time = cdata[self.cisco_Time]
        if self.check_size( cisco_date, 8, 'Cisco Date', linenumber ): return 1,0
        if self.check_size( cisco_time, 4, 'Cisco Time', linenumber ): return 1,0
        start_time = cisco_date[4:] + cisco_date[0:2] + cisco_date[2:4] + cisco_time + '00'
        vdata[ self.vid_StartTime ] = start_time
        
        #vid_Duration
        duration_min = int( cdata[self.cisco_Duration] )
        duration = '%02d%02d00'%(duration_min / 60, duration_min % 60)
        vdata[ self.vid_Duration ] = duration

        # vid_ProgramId
        # vdata[ self.vid_ProgramId ] = self.create_id_from_text(cdata[self.cisco_STitle])
        # Per V240P User Business Requirements, Section: DTA-F0315 and DTA-F0360
        vdata[ self.vid_ProgramId ] = str(EpisodeCounter)
        
        # vid_Title
        # ToDo: Do we want Short or Long title here?
        vdata[ self.vid_Title ] = cdata[self.cisco_STitle]
        
        # vid_Block1
        vid_ProgramType = 4
        if self.is_on( cdata, self.cisco_IsMovie): 
            vid_ProgramType = 2
        vid_Definition = 0
        if self.is_on( cdata, self.cisco_Hd): 
            vid_ProgramType = 1
        stereo   = self.is_on( cdata, self.cisco_Stereo)
        surround = self.is_on( cdata, self.cisco_Surround) 
        vid_AudioType = 0
        if stereo             : vid_AudioType = 1
        if surround           : vid_AudioType = 2
        if stereo and surround: vid_AudioType = 3
        vid_Captioned = self.is_on( cdata, self.cisco_ClosedCaptioned) 
        vid_Live      = self.is_on( cdata, self.cisco_Live)
        # We will assume no  Interactive
        vid_Interactive = 0
        vid_StarRating = min( 7, self.get_int( cdata, self.cisco_HalfStars) / 2 )
        # We will assume no  vid_Language = 1 (english)
        vid_Language = 1
        vdata[ self.vid_Block1 ] = '%d%d%d%d%d%d%d%d' %(vid_ProgramType,vid_Definition,vid_AudioType,
            vid_Captioned, vid_Live, vid_Interactive, vid_StarRating, vid_Language)
        
        # vid_Category
        vdata[ self.vid_Category ]    = ''
        vdata[ self.vid_Subcategory ] = ''
        theme = cdata[self.cisco_Theme].strip()
        parts = theme.split(',')
        n_cat = -2
        n_sub = -2
        if len(parts) > 1:
            cat = parts[0].strip()
            sub = parts[1].strip()
        else:
            cat = theme
            sub = ''
        if cat:
            try:
                n_cat = int( cat )
            except:
                n_cat = -1
        if sub:
            try:
                n_sub = int( sub )
            except:
                n_sub = -1
        if n_cat >= -1:
            if n_cat in self.cisco_to_videotron_themes:
                array = self.cisco_to_videotron_themes[n_cat]
                n_cat = array[0]
                if len(array) > 1:
                    n_sub = array[1]
            else:
                self.warning('Line %7d: Can not convert Theme <%s> to Category'%(
                    linenumber, theme))
                n_cat = -2
        if n_sub >= -1:
            if n_sub in self.cisco_to_videotron_themes:
                array = self.cisco_to_videotron_themes[n_sub]
                n_sub = array[0]
            else:
                self.warning('Line %7d: Can not convert Theme <%s> to Sub Category'%(
                    linenumber, theme))
                n_sub = -2
        if n_cat >= 0:
            vdata[ self.vid_Category    ] = '%d'%(n_cat)
        if n_sub >= 0:
            vdata[ self.vid_Subcategory ] = '%d'%(n_sub)
                    
        # vid_SeriesId
        #SeriesId = self.create_id_from_text(cdata[self.cisco_STitle])
        #SeriesId = self.create_id_from_text(cdata[self.cisco_LTitle])
        SeriesId = self.create_id_from_text(cdata[self.cisco_STitle])
        vdata[ self.vid_SeriesId ] = SeriesId
        # Per V240P User Business Requirements, Section: DTA-F0315 and DTA-F0360
        #vdata[ self.vid_SeriesId ] = ''
        
        # vid_SeriesId
        # Per V240P User Business Requirements, Section: DTA-F0315
        vdata[ self.vid_EpisodeTitle ] = '%s (%d)'%(cdata[self.cisco_LTitle],EpisodeCounter)
        
        # vid_EpisodeId
        vdata[ self.vid_EpisodeId ] = str(EpisodeCounter)
        
        # vid_Rating
        mpaa = cdata[self.cisco_MpaaRating]
        tv   = cdata[self.cisco_TvRating]
        if mpaa not in self.mpaa_ratings_available:
            self.warning('Line %7d: Un-known MPAA Rating <%s> should be one of: %s'%(
                linenumber, mpaa, str(self.mpaa_ratings_available)))
        if tv   not in self.tv_ratings_available:
            self.warning('Line %7d: Un-known TV Rating <%s> should be one of: %s'%(
                linenumber, mpaa, str(self.tv_ratings_available)))
        # Per V240P User Business Requirements, Section: DTA-F0320
        rating  = '8+'
        if tv   ==      'TVY':                    rating =  'G'
        if mpaa ==    'MPAAG':                    rating =  'G'
        if tv   ==     'TVY7':                    rating =  '8+'
        if mpaa ==   'MPAAPG':                    rating =  '8+'
        if mpaa == 'MPAAPG13':                    rating = '13+'
        if tv   ==    'TVV14':                    rating = '13+'
        if tv   ==     'TVMA':                    rating = '16+'
        if mpaa == 'MPAANC17':                    rating = '16+'
        if mpaa ==    'MPAAR':                    rating = '18+'
        if tv   ==       'NR':                    rating = 'NR'
        if mpaa ==       'NR':                    rating = 'NR'
        # Per assumptions 
        if tv   ==      'TVG':                    rating = 'G'
        if tv   ==     'TVPG':                    rating = '8+'
        if tv   == 'NR-ADULT':                    rating = '18+'
        if mpaa == 'NR-ADULT':                    rating = '18+'
        if self.is_on( cdata, self.cisco_Nudity): rating = '18+'
        vdata[ self.vid_Rating ] = rating
                
        # vid_ProductionYear
        vdata[ self.vid_ProductionYear ] = ''

        # vid_Sap
        vdata[ self.vid_Sap ] = str( self.is_on( cdata, self.cisco_Sap))

        # vid_Country
        vdata[ self.vid_Country ] = ''

        # vid_FullDescription
        vdata[ self.vid_FullDescription ] = cdata[self.cisco_LDesc]

        # vid_Actors
        vdata[ self.vid_Actors ] = ''

        return 0,vdata

    def print_ratings(self, cisco_data):
        self.histo_rating_mpaa = {}
        self.histo_rating_tv = {}
        for i, cdata in enumerate( cisco_data):
            mpaa = cdata[self.cisco_MpaaRating]
            tv   = cdata[self.cisco_TvRating]
            if mpaa not in self.histo_rating_mpaa:
                self.histo_rating_mpaa[mpaa] = 0
            self.histo_rating_mpaa[mpaa] += 1
            if tv not in self.histo_rating_tv:
                self.histo_rating_tv[tv] = 0
            self.histo_rating_tv[tv] += 1
        mpaas = self.histo_rating_mpaa.keys()
        mpaas.sort()
        print 'MPAA   Count:'
        for mpaa in mpaas:
            print '%-10s %d'%(mpaa, self.histo_rating_mpaa[mpaa])
        tvs = self.histo_rating_tv.keys()
        tvs.sort()
        print 'TV   Count:'
        for tv in tvs:
            print '%-10s %d'%(tv, self.histo_rating_tv[tv])

    def convert_content_to_videotron(self):
        #self.reset_conversion() # Call at start to include for multiple files
        self.cisco_data = self.parse(self.content_in, '|', self.cisco_parse_tags)
        self.content_in = None
        date_size = len(self.cisco_data)
        percent_reporting_on = date_size > 1000
        # disable percent_reporting feature as the conversions are qick
        percent_reporting_on = 0
        if percent_reporting_on:
            percent_reporting_10 = date_size/10
            percent_reporting_n  = percent_reporting_10 - 10
            percent_reporting_i  = 0
            self.info('  Input parsing done, %d program items'%(date_size))
        self.videotron_data = []
        for i, cdata in enumerate( self.cisco_data):
            if percent_reporting_on and percent_reporting_n < i:
                percent_reporting_n += percent_reporting_10
                percent_reporting_i += 1
                self.info('  Conversion %d%% done %d items ...'%(percent_reporting_i*10, i))
            error, vdata =  self.convert_one_cisco_program_to_videotron(cdata, i+1)
            if not error:
                self.videotron_data.append(vdata)
        if percent_reporting_on:
            self.info('  Output conversion ready with %d program items'%(len(self.videotron_data)))
            percent_reporting_n  = percent_reporting_10 - 10
            percent_reporting_i  = 0

        self.content_out = self.convert_to_videotron_text(self.videotron_data)
        self.videotron_data = []

    def convert_to_videotron_text(self, videotron_data):
        content_data = []
        for i, vdata in enumerate( videotron_data):
            text = ''
            for i, tag in enumerate( self.videotron_parse_tags):
                if i:
                    text += '|'
                text += vdata.get(tag, '')
                if tag not in vdata:
                    pass
            content_data.append(text)
        # Joining a list is many times faster than text += data.
        content_out  = '\n'.join(content_data) + '\n'
        return content_out



    notes = '''
    Real Videotron
    MLOUD||20110328180000|020000|219335E|Loud Videoflow|10010001|16|||||8+|0|||Music videos.|
    
    Converted Videotron
    AMC||20110328113000|003000|831203297101|Sexy Faces|10000001|0|0|8312111432108|Sexy Faces|8312111432108|Empty|0|||See how Lifestyle Lift has transformed the lives of nearly 100,000 people.|
    '''
                
    #####################################################################################
    #   Cisco / SA IPG Overlap and Time-slot Functions
    #####################################################################################
    def ipg_overlap_analysis(self, videotron_format=0):
        self.load_time_slots(videotron_format)
        # Look for Overlap
        for i, channel in enumerate( self.channel_list):
            slots = self.channel_data[channel]
            n     = len(slots)
            for j, slot1 in enumerate( slots):
                if j + 1 < n:
                    slot2 = slots[j + 1]
                    if slot1.end > slot2.start:
                        self.error('Overlap Error at lines %d and %d, an Overlap of %s :\n%7d %s\n%7d %s'%(
                            slot1.linenumber,
                            slot2.linenumber,
                            self.make_time_from_seconds( slot1.end - slot2.start),
                            slot1.linenumber,
                            self.parse_lines[slot1.p_index],
                            slot2.linenumber,
                            self.parse_lines[slot2.p_index]))

    def sort_by_start_time(self, slot_a, slot_b ):
        return( cmp(slot_a.start,slot_b.start))
    
    def load_time_slots(self, videotron_format=0):
        #self.reset_overlap() # Call at start to include for multiple files
        if videotron_format:
            self.ipg_data     = self.parse(self.content_in, '|', self.videotron_parse_tags,
                cisco_mode=0)
        else:
            self.ipg_data     = self.parse(self.content_in, '|', self.cisco_parse_tags)
            #self.print_ratings(self.ipg_data)
        #self.content_in     = None
        self.start_max = time.time() - 3600 * 24 * 365 * 3 # Three years past 
        self.start_min = time.time() + 3600 * 24 * 365 * 3 # Three years future
        # Create time slot objects
        self.slot_data = []
        channel_histo = {}
        error_count = 0
        for i, idata in enumerate( self.ipg_data):
            slot = TimeSlot(self)
            if videotron_format:
                slot.load_videotron_data(idata, i+1)
            else:
                slot.load_cisco_data(idata, i+1)
                
            self.start_min = min(self.start_min, slot.start)
            self.start_max = max(self.start_max, slot.start)
            if not slot.is_error:
                self.slot_data.append(slot)
                if slot.channel not in channel_histo:
                    channel_histo[slot.channel] = 0 
                channel_histo[slot.channel] += 1
            else:
                error_count += 1 
        self.channel_list = channel_histo.keys() 
        self.channel_list.sort()              
        self.report_info('Time slots: %d programs, %d channels from %s to %s'%(
            len(self.ipg_data), len(self.channel_list),
            time.ctime(self.start_min),
            time.ctime(self.start_max),
            ))
        if error_count:
            self.info('   !!!! %d Errors found'%(error_count))
            
        # Order time slot per channel and sort by start time
        self.channel_data = {}
        for i, slot in enumerate( self.slot_data):
            if slot.channel not in self.channel_data:
                self.channel_data[slot.channel] = []
            self.channel_data[slot.channel] += [slot]
        for i, channel in enumerate( self.channel_list):
            self.channel_data[channel].sort( self.sort_by_start_time )
    
    
    def make_time_windows(self, duration = 6*60*60): # Default 6 hours
        # Move into time windows
        verbose = 0
        self.windows = []
        for i, channel in enumerate( self.channel_list):
            slots = self.channel_data[channel]
            n     = len(slots)
            for j, slot1 in enumerate( slots):
                w_match = None
                for k, window in enumerate( self.windows):
                    if window.is_in_window( slot1.start ):
                        w_match = window
                        break
                if w_match:
                    w_match.slots.append(slot1)
                else:
                    window = TimeWindow(self, slot1.start, duration)
                    window.slots.append(slot1)
                    self.windows.append(window)
        for window in self.windows:
            self.report_info('Converting file %s:'%(window.acrname))
            window.compile_content()
        if verbose:
            for window in self.windows:
                window.print_descr()
        # Move into days
        self.days = []
        for window in self.windows:
            d_match = None
            for k, day in enumerate( self.days):
                if day.is_in_window( window.start ):
                    d_match = day
                    break
            if d_match:
                d_match.twindows.append(window)
            else:
                day = DayWindow(self, window.start)
                day.twindows.append(window)
                self.days.append(day)
        if len(self.days):
            self.days.sort( self.sort_by_day )
            day_offset = self.days[0].yday - self.days[0].wday
            for day in self.days:
                day.sort_windows()
                day.iday = day.yday - day_offset
        if verbose:
            for day in self.days:
                day.print_descr()
        for day in self.days:
            if len(day.twindows) != 4:
                self.warning('On Day %s, Instead of 4 Six-hour-windows, there were only %d'%(
                    day.day,len(day.twindows)))
                
    def sort_by_day(self, a, b):
        return(cmp(a.day, b.day))
                

    def save_days(self, dest_dir):
        if len(self.days):
            for day in self.days:
                day.save(dest_dir)

        


    

class DayWindow(object):

    def __init__(self, parent, start):
        self.parent    = parent
        self.duration  = 24*60*60
        self.start     = self.find_starting_time(start)
        self.end       = self.start + self.duration
        self.twindows  = []
        tm1            = time.localtime(self.start)
        self.day       = '%d%02d%02d'%(tm1[0],tm1[1],tm1[2])
        self.wday      = int( tm1[6])
        self.yday      = int( tm1[7])
        self.iday      = 0

    def is_in_window(self, start ):
        return( start >= self.start and start < self.end )

    def find_starting_time(self, start_in):
        tm1 = time.localtime(start_in)
        self.tm  = time.struct_time( (tm1[0],tm1[1],tm1[2],0,0,0,0,0,-1) )
        start    = int( time.mktime(self.tm) )
        return(start)

    def print_descr(self, mode =1):
        print 'Day %s WDay %d YDay %3d IDay %2d with %d windows'%(
            self.day,
            self.wday,
            self.yday,
            self.iday,
            len(self.twindows))
        
    def sort_by_start(self, a, b):
        return(cmp(a.start, b.start))
        
    def sort_windows(self):
        if len(self.twindows):
            self.twindows.sort( self.sort_by_start)
            
    def save(self, dest_dir):            
        # Minus 1 more per Josh request
        filename = os.path.join( dest_dir, 'epg_day_%d.zip'%(self.iday))
        self.arcnames = []
        self.contents = []
        for twindow in self.twindows:
            self.arcnames.append(twindow.acrname)
            self.contents.append(twindow.content)
        self.parent.save_content_list_as_zip(filename, self.arcnames, self.contents )
        return 0
        #self.ipg.convert_content_to_videotron()
        #LogControl.turn_gui_handler_on()
        #arc_name = os.path.basename(self.ipg.get_name_no_extension(file))
        #new_file = os.path.join( self.list_video.directory, arc_name + '.zip')
        #new_file = self.ipg.get_new_filename_dated_if_needed(new_file)
        #self.ipg.save_content_as_zip(new_file, arc_name)


    
class TimeWindow(object):
    
    def __init__(self, parent, start, duration):
        self.parent       = parent
        self.duration     = duration
        self.start        = self.find_starting_time( start, duration)
        self.end          = self.start + self.duration
        self.slots        = []
        tm1               = time.localtime(self.start)
        self.day          = '%d%02d%02d'%(tm1[0],tm1[1],tm1[2])
        self.time         = '%02d%02d'%(tm1[3],tm1[4])
        self.acrname   = 'epg-%s-%s'%(self.day, self.time)
        self.content_list = []
        self.content      = ''
    
        
    def is_in_window(self, start ):
        return( start >= self.start and start < self.end )


    def find_starting_time(self, start_in, duration):
        tm1 = time.localtime(start_in)
        duration_h = duration / 3600
        tm_hour = int ( tm1[3] / duration_h) * duration_h
        self.tm  = time.struct_time( (tm1[0],tm1[1],tm1[2],tm_hour,0,0,0,0,-1) )
        start    = int( time.mktime(self.tm) )
        return(start)

    
    def print_descr(self, mode =1):
        if mode == 2:
            print 'Window from %s to %s with %d programs'%(
                time.ctime(self.start),
                time.ctime(self.end),
                len(self.slots))
        else:
            print 'Window %s %s with %d programs'%(
                self.day,
                self.acrname,
                len(self.slots))


    def compile_content(self):
        self.cisco_data = []
        for slot in self.slots:
            self.cisco_data.append( slot.data  )
        self.videotron_data = []
        self.histo_rating_mpaa = {}
        self.histo_rating_tv = {}
        for i, cdata in enumerate( self.cisco_data):
            error, vdata =  self.parent.convert_one_cisco_program_to_videotron(cdata, i+1)
            if not error:
                self.videotron_data.append(vdata)
        self.content = self.parent.convert_to_videotron_text(self.videotron_data)
        self.cisco_data = []
        self.videotron_data = []



class TimeSlot(object):
    
    def __init__(self, parent):
        self.parent     = parent
        self.start      = 0
        self.end        = 0
        self.channel    = 0
        self.linenumber = 0
        self.is_error   = False
        self.p_index    = 0
        self.data       = None

    def load_cisco_data(self, cdata, linenumber ):
        self.linenumber = linenumber            
        self.data       = cdata
        self.cisco_date = cdata[self.parent.cisco_Date]
        self.cisco_time = cdata[self.parent.cisco_Time]
        self.cisco_dura = cdata[self.parent.cisco_Duration]
        self.channel    = cdata[self.parent.cisco_CName]
        self.p_index    = cdata[self.parent.parse_index]
        if self.parent.check_size( self.cisco_date, 8, 'Cisco Date', self.linenumber ): return 1,0
        if self.parent.check_size( self.cisco_time, 4, 'Cisco Time', self.linenumber ): return 1,0
        error    =  0
        tm_year  = -1 
        tm_mon   = -1
        tm_mday  = -1
        tm_hour  = -1
        tm_min   = -1
        duration = -1
        try:
            tm_year  = int(self.cisco_date[4:]) 
            tm_mon   = int(self.cisco_date[0:2]) 
            tm_mday  = int(self.cisco_date[2:4]) 
            tm_hour  = int(self.cisco_time[0:2]) 
            tm_min   = int(self.cisco_time[2:])
            duration = int(self.cisco_dura )
        except:
            self.is_error = True
        if self.is_error:
            if tm_mday < 0:
                self.error('Line %7d: Format error in Date <%s>'%(
                    linenumber, self.cisco_date))
            elif tm_min < 0:
                self.error('Line %7d: Format error in Time <%s>'%(
                    linenumber, self.cisco_time))
            else:
                self.error('Line %7d: Format error in Duration <%s>'%(
                    linenumber, self.cisco_dura))
        self.tm    = time.struct_time( (tm_year,tm_mon,tm_mday,tm_hour,tm_min,0,0,0,-1) )
        self.start = int( time.mktime(self.tm) )
        self.end   = self.start + duration * 60


    def load_videotron_data(self, vdata, linenumber ):
        self.linenumber     = linenumber            
        self.data           = vdata
        self.p_index        = vdata[self.parent.parse_index]
        self.channel        = vdata[self.parent.vid_VideotronCallLetters]
        self.videotron_time = vdata[self.parent.vid_StartTime]
        self.videotron_dura = vdata[self.parent.vid_Duration]
        if self.parent.check_size( self.videotron_time,14, 'Videotron Date'    ,
            self.linenumber ): return 1,0
        if self.parent.check_size( self.videotron_dura, 6, 'Videotron Duration',
            self.linenumber ): return 1,0
        error    =  0
        tm_year  = -1
        tm_mon   = -1
        tm_mday  = -1
        tm_hour  = -1
        tm_min   = -1
        tm_sec   = -1
        duration = -1
        try:
            tm_year   = int(self.videotron_time[  : 4]) 
            tm_mon    = int(self.videotron_time[ 4: 6]) 
            tm_mday   = int(self.videotron_time[ 6: 8]) 
            tm_hour   = int(self.videotron_time[ 8:10]) 
            tm_min    = int(self.videotron_time[10:12])
            tm_sec    = int(self.videotron_time[12:14])
            dura_hour = int(self.videotron_dura[  : 2])
            dura_min  = int(self.videotron_dura[ 2: 4])
            dura_sec  = int(self.videotron_dura[ 4: 6])
        except:
            self.is_error = True
        if self.is_error:
            if tm_min < 0:
                self.error('Line %7d: Format error in Videotron Time <%s>'%(
                    linenumber, self.videotron_time))
            else:
                self.error('Line %7d: Format error in Videotron Duration <%s>'%(
                    linenumber, self.videotron_dura))
        duration   = dura_hour * 3600 + dura_min * 60 + dura_sec
        self.tm    = time.struct_time( (tm_year,tm_mon,tm_mday,tm_hour,tm_min,tm_sec,0,0,-1) )
        self.start = int( time.mktime(self.tm) )
        self.end   = self.start + duration







#####################################################################################
#   CLI Functions
#####################################################################################
        
if __name__ == '__main__':
    global data_list    
    global pyScript    
    paths       = os.path.split( sys.argv[0] )
    pyScript    = paths[1]
    data_list = IpgDataList()
    usage=""
    parser = OptionParser(usage=usage)
    configuration = {}
    parser.add_option('-c',
                      '--cisco_file',
                      help='cisco/sa ipg filename')
    parser.add_option('-v',
                      '--videotron_file',
                      help='videotron_file format filename')
    parser.add_option('-o',
                      '--overlap',
                      help='Run Overlap Analysis instead of Conversion')
    parser.add_option('-w',
                      '--warnings',
                      help='Report Warnings')
    options, args = parser.parse_args()
    data_list.config_cisco_file     = options.cisco_file
    data_list.config_videotron_file = options.videotron_file
    data_list.config_overlap        = 0
    if options.overlap and options.overlap != '0':
        data_list.config_overlap    = 1
    data_list.config_report         = 0
    if options.warnings and options.warnings != '0':
        data_list.config_report     = 1
    errors = []
    if data_list.config_overlap:
        if not data_list.config_cisco_file and not data_list.config_videotron_file:
            errors.append('Either option -c or -v filename is needed for Overlap Analysis')
        if data_list.config_cisco_file and data_list.config_videotron_file:
            errors.append('Either option -c or -v filename is needed (not both!) for Overlap Analysis')
    else:
       if not data_list.config_cisco_file:
            errors.append('Option -c cisco filename needed for Conversion.')
       if not data_list.config_videotron_file:
            errors.append('Option -v videotron filename needed for Conversion.')
    if errors:
        print 'Errors found in CLI format:'
        for error in errors:
            print error
        print('Usage examples:')
        print('  %s -c C:\\vtwo00_sa_ipg_input.dat.gz -v C:\\epg-20110328-0000.zip'%pyScript)
        print('  %s -c C:\\vtwo00_sa_ipg_input.dat.gz -v C:\\epg-20110328-0000'%pyScript)
        print('  %s -r1 -c C:\\vtwo00_sa_ipg_input.dat.gz -v C:\\epg-20110328-0000'%pyScript)
        print('  %s -o1 -c C:\\vtwo00_sa_ipg_output.dat.gz'%pyScript)
        print('  %s -o1 -c C:\\vtwo00_sa_ipg_output.dat'%pyScript)
        print('  %s -o1 -v C:\\epg-20110328-0000'%pyScript)
        sys.exit()
    data_list.run_from_cli()
        
    
    
    


