#*****************************************************************************
#                      IPG Converter Version 0.5
#
#           Python script for IPG Data Conversion and Analysis 
#              written by Mark S. Adams, Cisco Lawrenceville
#
#                  Copyright 2011, Cisco Systems, Inc.
#
#                     Proprietary and Confidential
#         Unauthorized distribution or copying is prohibited
#                          All rights reserved
#
# No part of this computer software may be reprinted, reproduced or utilized
# in any form or by any electronic, mechanical, or other means, now known or
# hereafter invented, including photocopying and recording, or using any
# information storage and retrieval system, without permission in writing
# from Cisco Systems, Inc.
#*****************************************************************************

import os
import sys
import time
import logging
import shutil
import ipg_converter_cli
from pytronix.lib.general import Tags
from pytronix.lib.file import Filelod
from pytronix.lib.general import LogControl

class ConfigFile(Filelod.Filelod):
    def __init__(self):
        self.config_tags  = [
            'ipgconvertCiscoPath',
            'ipgconvertVideotronPath',
            'ipgconvertEditor',
            ]
        Filelod.Filelod.__init__ (self, log)
        self.add_tags(self.config_tags)
        self.default_config = [{
            self.ipgconvertCiscoPath:
                os.path.join( self.filelod_locator.auto_path, 'IPG_Files_Cisco'),
            self.ipgconvertVideotronPath:
                os.path.join( self.filelod_locator.auto_path, 'IPG_Files_Videotron'),
            }]
        self.set_file_name_only(self.filelod_locator.local_name_no_extension + '.cfg')
        if not self.exists():
            self.create_default_file()
        else:
            #log.info('Reading config file')
            error, obj = self.erLoadFile()
            if error:
                self.create_default_file()
            elif (not os.path.exists(self.get_dir_cisco())) or (
                not os.path.exists( self.get_dir_videotron())): 
                self.create_default_file()


    def create_default_file(self):
        self.set_dictionary_list(self.default_config)
        log.info('Creating default config file:')
        log.info('         %s'%self.filename)
        self.auto_save_file()

    
    # get and set functions
    def get_dir_cisco(self):
        return( self.get_item({
            self.tFilelodIndex: 0,
            self.tFilelodFindTag: self.ipgconvertCiscoPath,
            }))


    def set_dir_cisco(self, value):
        return( self.set_item({
            self.tFilelodIndex: 0,
            self.tFilelodFindTag: self.ipgconvertCiscoPath,
            }, value, save=1))


    def get_dir_videotron(self):
        return( self.get_item({
            self.tFilelodIndex: 0,
            self.tFilelodFindTag: self.ipgconvertVideotronPath,
            }))


    def set_dir_videotron(self, value):
        return( self.set_item({
            self.tFilelodIndex: 0,
            self.tFilelodFindTag: self.ipgconvertVideotronPath,
            }, value, save=1))


    def get_editor(self):
        return( self.get_item({
            self.tFilelodIndex: 0,
            self.tFilelodFindTag: self.ipgconvertEditor,
            }))


    def set_editor(self, value):
        return( self.set_item({
            self.tFilelodIndex: 0,
            self.tFilelodFindTag: self.ipgconvertEditor,
            }, value, save=1))


class MyFunctions(object):
    def __init__(self, log_in):
        global log
        log = log_in
        self.board = None
        self.filelod_locator = Filelod.FilelodLocator(__file__)
        self.config = ConfigFile()
        self.list_cisco = None
        self.list_video = None
        self.test_mode_quick = False
        
        
    def initialize(self):
        self.get_gui_objs()
        self.fun_refresh_lists(0)
        self.ipg = ipg_converter_cli.IpgDataList()
        self.ipg.log_obj = log

        
    def get_gui_objs(self):
        self.view = self.board.view_objs[0]
        self.list_cisco = self.view.list_1
        self.list_video = self.view.list_2
        
    
    def shorten_directory(self, directory):
        return(os.path.basename(directory))
        new_directory = directory
        dirs = []
        parts = os.path.split(directory)
        while parts[1]:
            dirs.append(parts[1])
            parts = os.path.split(parts[0])
        if len(dirs) > 5:
            new_directory = dirs[-1]
            for i, dir in enumerate( reversed(dirs)):
                if (i and i < 2) or ( i > len(dirs) - 2):
                    new_directory = os.path.join(new_directory, dir)
                if i == 3:
                    new_directory += '..'
        return new_directory
        

    def edit_text_file(self, file, textEditor=0 ):
        if not textEditor:
            textEditor = self.config.get_editor()
        if not os.path.exists( textEditor ):
            log.warning('Configured Text Editor not found:\n    %s'%(textEditor))
            return 1
        if not os.path.exists( file ):
            log.warning('File not found:\n    %s'%(file))
            return 2
        application    = os.path.split(textEditor)[1]
        pid   = os.spawnv(os.P_NOWAIT, textEditor, (application, file))
        return 0


    #####################################################################################
    #   Functions called by GUI in order left to right
    #####################################################################################

    def fun_overlap_run_analysis(self, event):
        self.list_cisco.get_selected_files()
        self.list_video.get_selected_files()
        if not len(self.list_cisco.selected_files) and not len(self.list_video.selected_files):
            log.warning('No selected files')
            return
        if 0: # Not Used
            text = 'This Cisco / SA IPG Overlap Analysis generally '+(
                'takes about 10 seconds per file. During this time, this application window ') + (
                'may turn dark or blank.\n\n Are you ready to start the analysis ?')
            yes = self.board.gui.DialogAreYouSure( text, 'Ready to Start Overlap Analysis?')
            if not yes:
                return
        self.ipg.reset_overlap()
        for file in self.list_cisco.selected_files:
            self.overlap_analysis(file, videotron_format=0)
        for file in self.list_video.selected_files:
            self.overlap_analysis(file, videotron_format=1)
        self.fun_refresh_lists(0)


    def overlap_analysis(self, file, videotron_format=0):
        start = time.time()
        ext = self.ipg.get_extension(file).lower()
        if ext == 'gz':
            if self.ipg.load_gzip_file(file):
                return 1
        elif ext == 'zip' :
            log.warning('  !!! Please do not select ".zip" files: %s.'%(
                os.path.basename(file)))
            return 1
        else:
            if self.ipg.load_text_file(file):
                return 1
        # Turn GUI logging off since it will be frozen for many minutes.
        LogControl.turn_gui_handler_off()
        self.ipg.ipg_overlap_analysis(videotron_format)
        LogControl.turn_gui_handler_on()
        duration = time.time() - start 
        no_error_text = 'Warnings (%d), Errors (%d)'%(
            self.ipg.overlap_warnings, self.ipg.overlap_errors)
        if not self.ipg.overlap_warnings and not self.ipg.overlap_errors:
            no_error_text = ', no warnings or errors.'
        log.info('Overlap Analysis made in %d minutes and %d seconds %s'%(
            duration / 60, duration % 60, no_error_text))
        if self.ipg.overlap_warnings:
            log.info('!!! There were %d Warnings, See Overlap Messages'%(
                self.ipg.overlap_warnings))
        if self.ipg.overlap_errors:
            log.info('!!! There were %d Errors, See Overlap Messages'%(
                self.ipg.overlap_errors))                
        

    def fun_overlap_see_messages(self, event):
        if len( self.ipg.overlap_list ):
            log.info('Overlap Report:')
            for line in self.ipg.overlap_list:
                if line[0].lower() == 'w':
                    log.warning(line)
                elif line[0].lower() == 'e':
                    log.error(line)
                else:
                    log.info(line)
            log.info('End of Overlap Report:')
        else:
            log.info('Overlap Report: No Warnings, No Errors.')


    def fun_overlap_save_messages(self, event):
        if len( self.ipg.overlap_list ):
            filename = self.ipg.get_name_no_extension(self.ipg.filename_source)
            filename = self.ipg.get_dated_filename(filename, '_Overlap') + '.txt'
            self.text_file = open(filename, 'wb')
            self.text_file.write('Overlap Report Date: %s\n'%(time.ctime()))
            for line in self.ipg.overlap_list:
                if line[0].lower() == 'w':
                    self.text_file.write('Warning: %s\n'%line)
                elif line[0].lower() == 'e':
                    self.text_file.write('Error: %s\n'%line)
                else:
                    self.text_file.write('Info: %s\n'%line)
            self.text_file.write('End of Overlap Report.')
            self.text_file.close()
            self.fun_refresh_lists(0)
        else:
            log.info('Overlap Report: No Warnings, No Errors.')


        

    def fun_configure_editor(self, event):
        newFile = self.board.gui.LoadFileDialog(os.path.dirname(self.config.get_editor()))
        if newFile:
            self.config.set_editor(newFile)
            log.info('Editor command configured to:\n    %s' % (newFile))


    def fun_conversion_cisco_to_videotron(self, event):
        self.list_cisco.get_selected_files()
        if not len(self.list_cisco.selected_files):
            log.warning('No selected files')
            return
        if 1: # Used
            text = 'This Cisco / SA IPG conversion into Videotron IPG data generally '+(
                'takes about 20 seconds per file. During this time, this application window ') + (
                'may turn dark or blank.\n\n Are you ready to start the conversion ?')
            yes = self.board.gui.DialogAreYouSure( text, 'Ready to Start Conversion?')
            if not yes:
                return
        self.ipg.reset_conversion()
        for file in self.list_cisco.selected_files:
            start = time.time()
            self.convert_to_videotron(file)
            duration = time.time() - start 
            no_error_text = 'Warnings (%d), Errors (%d)'%(
                self.ipg.conversion_warnings, self.ipg.conversion_errors)
            if not self.ipg.conversion_warnings and not self.ipg.conversion_errors:
                no_error_text = ', no warnings or errors.'
            log.info('Converted in %d minutes and %d seconds %s'%(
                duration / 60, duration % 60, no_error_text))
            if self.ipg.conversion_warnings:
                log.info('!!! There were %d Warnings, See Conversion Messages'%(
                    self.ipg.conversion_warnings))
            if self.ipg.conversion_errors:
                log.info('!!! There were %d Errors, See Conversion Messages'%(
                    self.ipg.conversion_errors))
        self.fun_refresh_lists(0)


    def convert_to_videotron(self, file):
        ext = self.ipg.get_extension(file).lower()
        if ext == 'gz':
            self.ipg.load_gzip_file(file)
        elif  ext == 'txt' or  ext == 'dat':
            self.ipg.load_text_file(file)
        else:
            log.warning('  !!! Please only select files with ".gz",' + (
                '".dat", or ".txt" for this operation.' ))
            return 1
        # Turn GUI logging off since it will be frozen for many minutes.
        LogControl.turn_gui_handler_off()
        self.ipg.load_time_slots(videotron_format=0)
        self.ipg.make_time_windows()
        self.ipg.save_days(self.list_video.directory)
        LogControl.turn_gui_handler_on()
        
        
        
        if 0:
            self.ipg.convert_content_to_videotron()
            LogControl.turn_gui_handler_on()
            arc_name = os.path.basename(self.ipg.get_name_no_extension(file))
            new_file = os.path.join( self.list_video.directory, arc_name + '.zip')
            new_file = self.ipg.get_new_filename_dated_if_needed(new_file)
            self.ipg.save_content_as_zip(new_file, arc_name)


    def fun_conversion_see_messages(self, event):
        if len( self.ipg.conversion_list ):
            log.info('Conversion Report:')
            for line in self.ipg.conversion_list:
                if line[0].lower() == 'w':
                    log.warning(line)
                elif line[0].lower() == 'e':
                    log.error(line)
                else:
                    log.info(line)
            log.info('End of Conversion Report.')
        else:
            log.info('Conversion Report: No Warnings, No Errors.')


    def fun_set_dir_cisco(self, event):
        newDir = self.board.gui.LoadDirDialog(self.list_cisco.directory,
            "Choose directory for Test Scripts:")
        if newDir:
            self.config.set_dir_cisco(newDir)
            self.fun_refresh_lists(0)
            log.info('Cisco Directory set to: %s'%(newDir))

    
    def fun_ungz_file(self, event):
        self.list_cisco.get_selected_files()
        for file in self.list_cisco.selected_files:
            log.info('Un GZip: %s'%os.path.basename(file))
            if self.ipg.get_extension(file).lower() != 'gz':
                log.warning('  !!! Please only select files with ".gz" for this operation.' )
                continue
            self.ipg.load_gzip_file(file)
            ext = 'txt'
            if self.ipg.get_extension( self.ipg.get_name_no_extension(file)) == 'dat':
                ext = ''
            new_file = self.ipg.get_new_filename_dated_if_needed(
                self.ipg.get_name_no_extension(file), ext)
            self.ipg.content_out = self.ipg.content_in
            self.ipg.save_content_as_text(new_file)
            self.fun_refresh_lists(0)
            
    
    def fun_gz_file(self, event):
        self.list_cisco.get_selected_files()
        for file in self.list_cisco.selected_files:
            log.info('   GZip: %s'%os.path.basename(file))
            ext = self.ipg.get_extension(file).lower()
            if ext != 'txt' and ext != 'dat':
                log.warning('  !!! Please only select files with ".txt" or ".dat" for this operation.' )
                continue
            self.ipg.load_text_file(file)
            new_file = self.ipg.get_new_filename_dated_if_needed(file, 'gz')
            self.ipg.content_out = self.ipg.content_in
            self.ipg.save_content_as_gzip(new_file)
            self.fun_refresh_lists(0)


    def fun_print_filename(self, event):
        for file in self.list_cisco.get_selected_files() + self.list_video.get_selected_files():
            log.info(os.path.basename(file))

    
    def fun_conversion_save_messages(self, event):
        if len( self.ipg.conversion_list ):
            filename = self.ipg.get_name_no_extension(self.ipg.filename_source)
            filename = self.ipg.get_dated_filename(filename, '_Conversion') + '.txt'
            self.text_file = open(filename, 'wb')
            self.text_file.write('Conversion Report: Date: %s\n'%(time.ctime()))
            for line in self.ipg.conversion_list:
                if line[0].lower() == 'w':
                    self.text_file.write('Warning: %s\n'%line)
                elif line[0].lower() == 'e':
                    self.text_file.write('Error: %s\n'%line)
                else:
                    self.text_file.write('Info: %s\n'%line)
            self.text_file.write('End of Conversion Report:')
            self.text_file.close()
            self.fun_refresh_lists(0)
        else:
            log.info('Conversion Report: No Warnings, No Errors.')

    
    def fun_set_dir_videotron(self, event):
        newDir = self.board.gui.LoadDirDialog(self.list_video.directory,"Choose directory for Test Scripts:")
        if newDir:
            self.config.set_dir_videotron(newDir)
            self.fun_refresh_lists(0)
            log.info('Videotron Directory set to: %s'%(newDir))

    
    def fun_unzip_file(self, event):
        self.list_video.get_selected_files()
        for file in self.list_video.selected_files:
            log.info('Un  Zip: %s'%os.path.basename(file))
            if self.ipg.get_extension(file).lower() != 'zip':
                log.warning('  !!! Please only select files with ".zip" for this operation.' )
                continue
            self.ipg.extractall_zip_file(file)
            self.fun_refresh_lists(0)

    
    def fun_zip_file(self, event):
        self.list_video.get_selected_files()
        if not len(self.list_video.selected_files):
            log.warning('  No files on Videotron side selected.' )
            return 1        
        new_file = os.path.join( os.path.dirname(self.list_video.selected_files[0]),
            self.ipg.get_dated_filename('archive.zip'))
        log.info(new_file)
        self.ipg.save_files_as_zip( new_file, self.list_video.selected_files)
        self.fun_refresh_lists(0)
    

    def fun_edit_file(self, event):
        for file in self.list_cisco.get_selected_files() + self.list_video.get_selected_files():
            self.edit_text_file(file)


    def fun_rename_file(self, event):
        for file in self.list_cisco.get_selected_files() + self.list_video.get_selected_files():
            newValue = self.board.gui.TextEntryDialog('Please rename file.',
                'Rename File', os.path.basename(file))
            if newValue:
                newName = os.path.join( os.path.dirname(file), newValue)
                try:
                    os.rename(file, newName)
                except:
                    pass
        self.fun_refresh_lists(0)
    

    def fun_refresh_lists(self, event):
        self.list_cisco.directory = self.config.get_dir_cisco()
        self.list_video.directory = self.config.get_dir_videotron()
        self.list_cisco.list_obj.set_column_text(0, 'Cisco: '+
            self.shorten_directory( self.list_cisco.directory))
        self.list_video.list_obj.set_column_text(0, 'Videotron: '+
            self.shorten_directory(self.list_video.directory))        
        self.list_cisco.refresh_directory()
        self.list_video.refresh_directory()

    
    def fun_copy_file(self, event):
        for file in self.list_cisco.get_selected_files() + self.list_video.get_selected_files():
            new_file = self.ipg.get_new_filename_dated_if_needed(file)
            shutil.copyfile(file, new_file)
        self.fun_refresh_lists(0)


    def fun_delete_file(self, event):
        for file in self.list_cisco.get_selected_files() + self.list_video.get_selected_files():
            yes = self.board.gui.DialogAreYouSure( 'Delete %s?'%(os.path.basename(file)), 'DELETE FILE !!!')
            if not yes:
                continue
            try:
                os.remove(file)
            except:
                pass
        self.fun_refresh_lists(0)


    def open_url(self, url):
        import webbrowser
        webbrowser.open(url)


    def fun_help(self, event):
        internal = 'Internal Cisco Site'
        external = 'External Internet Site'
        newValue = self.board.gui.SingleChoiceDialog('Please select a Help Web Page.',
        ' Help Web Page Selection', 0, [internal,external])
        if newValue:
            if newValue == internal:
                self.open_url('http://wikicentral.cisco.com/display/GROUP/SPVTG+DOCSIS+IPG+Data+Converter')
            else:
                self.open_url('http://irtek.wikidot.com/ipg')


    def fun_clear_bottom_output(self, event):
        self.board.gui.loggerClear()
        #self.ipg.find_starting_time(6*60*60, time.time())

    
    
    note_used = '''
    def fun_test_mode_quick(self, event):
        # Not used.
        return
        if not self.test_mode_quick:
            text = 'Clicking Yes turns Test-Mode-Quick On, In this "Demo" mode, '+(
                'only the first few IPG program items are converted. \n\n') +(
                'Do you want Test-Mode-Quick turned On?')
            yes = self.board.gui.DialogAreYouSure( text, 'Turn Test-Mode-Quick is On?')
            if yes:
                self.test_mode_quick = not self.test_mode_quick
                self.ipg.test_mode_quick = self.test_mode_quick
        else:
            self.test_mode_quick = not self.test_mode_quick
            self.ipg.test_mode_quick = self.test_mode_quick
        if self.test_mode_quick:
            log.info('Test Mode Quick is On, Ready for Demos Only!.')
        else:
            log.info('Test Mode Quick is Off, Ready for normal operation.')

        
    def fun_test1(self, event):
        log.info('Test 1 not used.')

        
    def fun_test2(self, event):
        log.info('Test 2 not used.')
    '''
